#include "globals.hh"

#include <iostream>
#include <fstream>

using namespace std;

// implemented in other files
class WordReader;
class CharReader;

template<class reader>
int analyze(unsigned int n, istream &is, ostream &os);
int generate(unsigned int n, istream &is, ostream &os);

void educate_user()
{
  cerr << "usage:" << endl 
       << "  markov-o-mat <action> <n> <infile> <outfile>" << endl
       << endl
       << "where <action> is one of" << endl
       << "  analyze-char\tmodel <infile> as a <n>th order character markov source"<< endl 
       << "  analyze-word\tmodel <infile> as a <n>th order word markov source" << endl
       << "  generate\tgenerate <n> symbols of text from model given in <infile>" << endl << endl;
  exit(error_code_usage);
}

int main(int argc, char *argv[])
{
  // if we don't have 4 arguments, print usage, bail out.
  if (argc != 5)
    educate_user();

  // user parameters
  unsigned int n;
  ifstream is(argv[3]);
  ofstream os(argv[4]);

  // convert parameter 2 to integer
  char *test;
  n = strtol(argv[2], &test, 10);
  if (test == argv[2])
      // conversion failed miserably
      educate_user();

  // start appropriate function
  int retval;
  if (!strcmp(argv[1], "analyze-char"))
    retval = analyze<CharReader>(n, is, os);
  else if (!strcmp(argv[1], "analyze-word"))
    retval = analyze<WordReader>(n, is, os);
  else if (!strcmp(argv[1], "generate"))
    retval = generate(n, is, os);
  else
    educate_user();

  return retval;
}
