import java.io.BufferedReader;

import java.io.FileReader;

import java.io.IOException;

import java.io.Reader;



/*

 * MarkovOMat.java, Project MarkovOMat

 * 

 * Created on Nov 16, 2004 by willema

 */



/**

 * MarkovOMat

 * @author willema

 */

public class MarkovOMat {



    public static void main(String[] args) {

        

        if (args.length != 4) printUsage();

        

        String type = args[0];

        int order = Integer.parseInt(args[1]);

        String infile = args[2];

        String outfile = args[3];

        boolean charwise = true;

        

        if (!(type.equals("char") || type.equals("word"))) printUsage();

        if (type.equals("word")) charwise = false;

        if (order < 0) printUsage();

        

        System.out.println("Starting analysis of '"+infile+"'...");

        SymbolCounter s = analyze(order, charwise, infile);

				SymbolCounter pre = null;

				if (order > 0) pre = analyze(order-1, charwise, infile);

        if(s == null) {

            System.out.println("Analysis failed!");

            System.exit(1);

        } else {

						if (pre == null) s.writeToFile(outfile, false);

						else s.writeToFile(outfile, pre);

            System.out.println("Analysis successfully written to '"+outfile+"'!");

            if (order == 0) System.out.println("Entropy of the Markov-Quelle: "+s.getEntropy()+" bits");

        }

    }

    

    public static void printUsage() {

        System.out.println("Usage: MarkovOMat ('char'|'word') <order> <infile> <outfile>");

        System.exit(1);

    }

    

    public static SymbolCounter analyze(int order, boolean charwise, String filename) {

        String[] symbols = new String[order+1];

        int accepted = 0;

        SymbolCounter counter = new SymbolCounter();

        try {

            BufferedReader file = new BufferedReader(new FileReader(filename));

            String symbol;

            for(int i=0; i<order+1; i++) {

                symbols[i] = readNextSymbol(file, charwise);

            }

            counter.addSymbol(toString(symbols));

            while((symbol = readNextSymbol(file, charwise)) != "") {

                for(int i=0; i<order; i++) {

                    symbols[i] = symbols[i+1];

                }

                symbols[order] = symbol;

                counter.addSymbol(toString(symbols));

            }

						file.close();

            return counter;

        } catch(IOException e) {

            return null;

        }

    }

    

    public static String readNextSymbol(Reader file, boolean charwise) {

        String s = "";

        try {

            for(int i=file.read();;i=file.read()) {

                char c = (char)i;

                if (i == -1) return s;

                if (charwise && !Character.isLetter(c) && !Character.isSpaceChar(c)) continue;

                if (!charwise && s != "" && !Character.isLetter(c)) return s;

                else if (!charwise && s == "" && !Character.isLetter(c)) continue;

                s += c;

                s = s.toLowerCase();

                if (charwise) return s;

            }

        } catch (IOException e) {

            return null;

        }

 

    }

    

    public static String toString(Object[] array) {

        String s = "";

        for(int i=0; i<array.length; i++) {

            s += array[i].toString() + "_";

        }

        return s.substring(0, s.length()-1);

    }

    

}

